__title__ = 'ssaw'
__version__ = '0.1.1'
__description__ = 'Survey Solutions API Wrapper'
__url__ = 'https://github.com/vavalomi/ssaw'
__author__ = 'Zurab Sajaia'
__author_email__ = 'vavalomi@hotmail.com'



import json
import os
import re

from sgqlc.endpoint.requests import RequestsEndpoint

from .exceptions import NotAcceptableError, NotFoundError
from .headquarters import Client


class HQBase(object):
    _apiprefix: str = ""

    def __init__(self, client: Client) -> None:
        self._hq = client
        self.endpoint = RequestsEndpoint(client.baseurl + '/graphql', session=client.session)

    @property
    def url(self) -> str:
        return self._hq.baseurl + self._apiprefix

    def _make_call(self, method: str, path: str, filepath: str = None, parser=None, **kwargs):
        response = self._hq.session.request(method=method, url=path, **kwargs)
        rc = response.status_code
        if rc < 300:
            if method == 'get':
                if 'application/json' in response.headers['Content-Type']:
                    if parser:
                        return parser(response.content)
                    else:
                        return json.loads(response.content)

                elif 'application/zip' in response.headers['Content-Type']:
                    return self._get_file_stream(filepath, response)
            else:
                return json.loads(response.content) if response.content else True
        elif rc == 404:
            raise NotFoundError(response.text)
        elif rc in [400, 406]:
            raise NotAcceptableError(response.text)
        else:
            response.raise_for_status()

    @staticmethod
    def _get_file_stream(filepath, response):
        d = response.headers['content-disposition']
        fname = re.findall(
            r"filename[ ]*=([^;]+)", d, flags=re.IGNORECASE)

        if not fname:
            fname = re.findall(
                r"filename\*=utf-8''(.+)", d, flags=re.IGNORECASE)

        fname = fname[0].strip().strip('"')
        outfile = os.path.join(filepath, fname)

        with open(outfile, 'wb') as f:
            for chunk in response.iter_content(chunk_size=1024):
                if chunk:
                    f.write(chunk)
        return outfile



class NotFoundError(Exception):
    def __init__(self, objecttype):
        self.expression = objecttype + ' not found'
        self.message = objecttype + ' not found'


class IncompleteQuestionnaireIdError(Exception):
    def __init__(self):
        self.expression = 'Invalid questionnaire id'
        self.message = 'Both questionnaire guid and version number must be specified'


class UnauthorizedError(Exception):
    def __init__(self):
        self.expression = 'Unauthorized'
        self.message = 'Either username or password were not provided or wrong'


class NotAcceptableError(Exception):
    def __init__(self, message):
        self.expression = 'Not Acceptable'
        self.message = message


class GraphQLError(Exception):
    def __init__(self, message):
        self.expression = 'GraphQL Error'
        self.message = message



from requests import Session

from .__about__ import __title__, __version__


class Client(object):
    """Initializes the API client

    Parameters
    ----------
    url: str
        URL of the headquarters app
    api_user: str
        API user name
    api_password: str
        API user password
    """

    def __init__(self, url: str, api_user: str, api_password: str):
        session = Session()
        session.auth = (api_user, api_password)
        signature = 'python-{}/{}'.format(__title__, __version__)
        session.headers.update({'User-Agent': signature})
        self.baseurl = url.rstrip("/")
        self.session = session



from .utils import to_hex, to_qidentity


class Assignment(object):
    def __init__(self, responsible, quantity, questionnaire_id,
                 identifying_data=None, email='', password='', webmode=False,
                 audio_recording_enabled=False, comments=''):
        """[summary]

        Parameters
        ----------
        responsible : [type]
            [description]
        quantity : [type]
            [description]
        questionnaire_id : [type]
            [description]
        email : str, optional
            [description], by default ''
        password : str, optional
            [description], by default ''
        webmode : bool, optional
            [description], by default False
        audio_recording_enabled : bool, optional
            [description], by default False
        comments : str, optional
            [description], by default ''
        """

        self.responsible = responsible
        self.quantity = quantity
        self.questionnaire_id = questionnaire_id
        self.identifying_data = identifying_data
        self.email = email
        self.password = password
        self.webmode = webmode
        self.audio_recording_enabled = audio_recording_enabled
        self.comments = comments

    def __str__(self):
        return(str(self.__dict__))

    @classmethod
    def from_dict(cls, dict):
        obj = cls(
            responsible=dict['ResponsibleName'],
            quantity=dict['Quantity'],
            questionnaire_id=dict['QuestionnaireId'],
            identifying_data=dict['IdentifyingData'] if 'IdentifyingData' in dict else [
            ],
            email=dict['Email'],
            password=dict['Password'],
            webmode=dict['WebMode']
        )
        setattr(obj, 'id', dict['Id'])
        setattr(obj, 'responsible_id', dict['ResponsibleId'])
        setattr(obj, 'interviews_count', dict['InterviewsCount'])
        setattr(obj, 'archived', dict['Archived'])
        setattr(obj, 'created_utc', dict['CreatedAtUtc'])
        setattr(obj, 'updated_utc', dict['UpdatedAtUtc'])
        if 'IsAudioRecordingEnabled' in dict:
            setattr(obj, 'audio_recording_enabled',
                    dict['IsAudioRecordingEnabled'])
        return obj

    def to_json(self):
        return {
            "Responsible": self.responsible,
            "Quantity": self.quantity,
            "QuestionnaireId": self.questionnaire_id,
            "Email": self.email,
            "Password": self.password,
            "WebMode": self.webmode,
            "IsAudioRecordingEnabled": self.audio_recording_enabled,
            "Comments": self.comments,
            "IdentifyingData": self.identifying_data,
        }


class QuestionnaireListItem(object):
    def __init__(self, dict):
        self.questionnaire_identity = dict['QuestionnaireIdentity']
        self.questionnaire_id = dict['QuestionnaireId']
        self.version = dict['Version']
        self.title = dict['Title']
        self.variable = dict['Variable']
        self.last_entry_date = dict['LastEntryDate']

    def __str__(self):
        return(str(self.__dict__))

    @classmethod
    def from_dict(cls, dict):
        return cls(dict)


class ExportJob(object):
    def __init__(self,
                 questionnaire_identity,
                 export_type='Tabular',
                 interview_status='All',
                 from_date=None,
                 to_date=None,
                 access_token=None,
                 storage_type=None):
        """[summary]

        Parameters
        ----------
        questionnaire_identity : [type]
            [description]
        export_type : str, optional
            [description], by default 'Tabular'
        interview_status : str, optional
            [description], by default 'All'
        from_date : [type], optional
            [description], by default None
        to_date : [type], optional
            [description], by default None
        access_token : [type], optional
            [description], by default None
        storage_type : [type], optional
            [description], by default None
        """

        if type(questionnaire_identity) is tuple:
            (questionnaire_id, questionnaire_version) = questionnaire_identity
            questionnaire_identity = to_qidentity(
                questionnaire_id, questionnaire_version)

        self.questionnaire_identity = questionnaire_identity
        self.export_type = export_type
        self.interview_status = interview_status
        self.from_date = from_date
        self.to_date = to_date
        self.access_token = access_token
        self.storage_type = storage_type
        self._export_status: str
        self._has_export_file: bool
        self._cancel_link: str
        self._download_link: str

    @property
    def export_status(self) -> str:
        return self._export_status

    @property
    def has_export_file(self) -> bool:
        return self._has_export_file

    @property
    def cancel_link(self) -> str:
        return self._cancel_link

    @property
    def download_link(self) -> str:
        return self._download_link

    def __str__(self) -> str:
        return(str(self.__dict__))

    @classmethod
    def from_dict(cls, dict):
        obj = cls(
            export_type=dict['ExportType'],
            questionnaire_identity=dict['QuestionnaireId'],
            interview_status=dict['InterviewStatus'],
            from_date=dict['From'] if 'From' in dict else None,
            to_date=dict['To'] if 'To' in dict else None,
            access_token=dict['AccessToken'] if 'AccessToken' in dict else None,
            storage_type=dict['StorageType'] if 'StorageType' in dict else None
        )
        setattr(obj, 'job_id', dict['JobId'])
        setattr(obj, '_export_status', dict['ExportStatus'])
        setattr(obj, 'start_date', dict['StartDate'])
        setattr(obj, 'complete_date', dict['CompleteDate'])
        setattr(obj, 'progress', dict['Progress'])
        if 'ETA' in dict:
            setattr(obj, 'eta', dict['ETA'])
        if 'Links' in dict:
            if 'Cancel' in dict['Links']:
                setattr(obj, '_cancel_link', dict['Links']['Cancel'])
            if 'Download' in dict['Links']:
                setattr(obj, '_download_link', dict['Links']['Download'])
        setattr(obj, '_has_export_file', dict['HasExportFile'])
        return obj

    def to_json(self):
        ret = {
            "ExportType": self.export_type,
            "QuestionnaireId": self.questionnaire_identity,
            "InterviewStatus": self.interview_status,
            "From": self.from_date,
            "To": self.to_date,
            "AccessToken": self.access_token,
            "StorageType": self.storage_type,
        }
        return {k: v for k, v in ret.items() if v is not None}


class InterviewAnswers(object):
    def __init__(self, answers: dict):
        data = {}
        variables = {}
        self._raw_data = answers["Answers"]
        for ans in self._raw_data:
            val = ans["Answer"]
            id = to_hex(ans["QuestionId"]["Id"])
            variables[ans["VariableName"]] = id
            if val is None:
                continue
            if len(ans["QuestionId"]["RosterVector"]) > 0:
                key = "_".join([id, ] + [str(r)
                                         for r in ans["QuestionId"]["RosterVector"]])
            else:
                key = id
            data[key] = val
        self._data = data
        self._variables = variables

    def __str__(self) -> str:
        return(str(self._raw_data))

    def get_answer(self, variable: str = None, question_id: str = None, roster_vector: list = []):
        if variable:
            if variable in self._variables:
                key = self._variables[variable]
            else:
                raise TypeError("get_answer() variable not found")
        else:
            if question_id:
                key = to_hex(question_id)
            else:
                raise TypeError(
                    "get_anwer() either 'variable' or 'question_id' argument is required")

        if roster_vector:
            key = "_".join([key, ] + [str(r) for r in roster_vector])

        if key in self._data:
            return self._data[key]
        else:
            return None

    def __iter__(self):
        return iter(self._raw_data)



QUESTION_TYPES = {
    0: 'SingleQuestion',
    3: 'MultyOptionsQuestion',
    4: 'NumericQuestion',
    5: 'DateTimeQuestion',
    6: 'GpsCoordinateQuestion',
    7: 'TextQuestion',
    9: 'TextListQuestion',
    10: 'QRBarcodeQuestion',
    11: 'MultimediaQuestion'
}

VARIABLE_TYPES = {
    3: 'Boolean'
}

ROSTER_TYPES = {
    1: 'Fixed set of items'
}


CLASSTYPE_MACRO = "System.Collections.Generic.Dictionary`2[[System.Guid, mscorlib],[WB.Core.SharedKernels.SurveySolutions.Documents.Macro, WB.Core.SharedKernels.Questionnaire]], mscorlib"  # noqa: E501
CLASSTYPE_LOOKUPTABLE = "System.Collections.Generic.Dictionary`2[[System.Guid, mscorlib],[WB.Core.SharedKernels.SurveySolutions.Documents.LookupTable, WB.Core.SharedKernels.Questionnaire]], mscorlib"  # noqa: E501



from typing import Iterator

from .base import HQBase
from .models import Assignment
from .utils import to_qidentity


class AssignmentsApi(HQBase):
    _apiprefix = "/api/v1/assignments"

    def get_list(self, questionnaire_id: str = None, questionnaire_version: int = None) -> Iterator[Assignment]:
        """Get list of assignments

        Parameters
        ----------
        questionnaire_id : str, optional
            Filter by specific questionnaire id
        questionnaire_version : int, optional
            Filter by specific version number

        Yields
        -------
        list of :class:`ssaw.models.Assignment` objects
        """
        path = self.url
        limit = 10
        offset = 1
        total_count = 11
        params = {
            'offset': offset,
            'limit': limit
        }
        if questionnaire_id and questionnaire_version:
            params['questionnaireId'] = to_qidentity(questionnaire_id, questionnaire_version)

        while offset < total_count:
            params['offset'] = offset
            r = self._make_call('get', path, params=params)
            total_count = r['TotalCount']
            offset += limit
            for item in r['Assignments']:
                yield Assignment.from_dict(item)

    def get_info(self, id: int) -> Assignment:
        """Get single assignment details

        Parameters
        ----------
        id : int
            Assignment Id

        Returns
        -------
            :class:`ssaw.models.Assignment` object
        """
        path = self.url + "/{}".format(id)
        item = self._make_call("get", path)
        return Assignment.from_dict(item)

    def create(self, obj: Assignment) -> Assignment:
        """Create new assignment

        Parameters
        ----------
        obj : Assignment
            Assignment object to be created

        Returns
        -------
            :class:`ssaw.models.Assignment` object
        """
        path = self.url
        res = self._make_call("post", path, json=obj.to_json())
        return Assignment.from_dict(res['Assignment'])

    def archive(self, id: int):
        """Archive assignment

        Parameters
        ----------
        id : int
            Assignment Id
        """
        path = self.url + "/{}/archive".format(id)
        self._make_call("patch", path)

    def unarchive(self, id: int):
        """Unarchive assignment

        Parameters
        ----------
        id : int
            Assignment Id
        """
        path = self.url + "/{}/unarchive".format(id)
        self._make_call("patch", path)

    def assign(self, id: int, responsible: str) -> Assignment:
        """Assign new responsible person for assignment

        Parameters
        ----------
        id : int
            Assignment Id
        responsible : str
            Username of the new responsible

        Returns
        -------
            Modified :class:`ssaw.models.Assignment` object
        """
        path = self.url + "/{}/assign".format(id)
        res = self._make_call("patch", path, json={"Responsible": responsible})
        return Assignment.from_dict(res)

    def get_quantity_settings(self, id: int) -> bool:
        """Checi if quantity may be edited for the assignment

        Parameters
        ----------
        id : int
            Assignment Id

        Returns
        -------
        bool
            `True` if quantity can be edited, `False` otherwise
        """
        path = self.url + "/{}/assignmentQuantitySettings".format(id)
        res = self._make_call("get", path)
        return res['CanChangeQuantity']

    def update_quantity(self, id: int, quantity: int) -> Assignment:
        """Change maximum quantity of interviews to be created

        Parameters
        ----------
        id : int
            Assignment Id
        quantity : int
            new quantity of interviews to be collected

        Returns
        -------
            Modified :class:`ssaw.models.Assignment` object

        """
        if not isinstance(quantity, int):
            raise TypeError('quantity must be a number')
        path = self.url + "/{}/changequantity".format(id)
        headers = {"Content-Type": "application/json-patch+json"}
        res = self._make_call("patch", path, data=str(quantity), headers=headers)
        return Assignment.from_dict(res)

    def close(self, id: int):
        """Close assignment by setting Size to the number of collected interviews

        Parameters
        ----------
        id : int
            Assignment Id
        """
        path = self.url + "/{}/close".format(id)
        self._make_call("post", path)

    def get_history(self, id: int) -> Iterator[dict]:
        page_size = 10
        start = 1
        total_count = 11
        params = {
            'start': start,
            'length': page_size
        }
        path = self.url + "/{}/history".format(id)
        while start < total_count:
            params['start'] = start
            r = self._make_call('get', path, params=params)
            total_count = r['RecordsFiltered']
            start += page_size
            yield from r['History']

    def get_recordaudio(self, id: int) -> bool:
        """Get status of audio recording for the assignment

        Parameters
        ----------
        id : int
            Assignment Id

        Returns
        -------
        bool
            True if audio recording is enabled, False otherwise
        """
        path = self.url + "/{}/recordAudio".format(id)
        res = self._make_call("get", path)
        return res['Enabled']

    def update_recordaudio(self, id: int, enabled: bool):
        """Turn recording of audio for the assignment

        Parameters
        ----------
        id : int
            Assignment Id
        enabled : bool
            True to turn audio recording on, False to turn it off
        """
        if not isinstance(enabled, bool):
            raise TypeError('enabled must be either True or False')
        path = self.url + "/{}/recordAudio".format(id)
        self._make_call("patch", path, json={'Enabled': enabled})



import sgqlc.types
import sgqlc.types.datetime


headquarters_schema = sgqlc.types.Schema()



########################################################################
# Scalars and Enumerations
########################################################################
class ApplyPolicy(sgqlc.types.Enum):
    __schema__ = headquarters_schema
    __choices__ = ('BEFORE_RESOLVER', 'AFTER_RESOLVER')


Boolean = sgqlc.types.Boolean

DateTime = sgqlc.types.datetime.DateTime

class Decimal(sgqlc.types.Scalar):
    __schema__ = headquarters_schema


Float = sgqlc.types.Float

ID = sgqlc.types.ID

Int = sgqlc.types.Int

class InterviewActionFlags(sgqlc.types.Enum):
    __schema__ = headquarters_schema
    __choices__ = ('CANBEREASSIGNED', 'CANBEDELETED', 'CANBEAPPROVED', 'CANBEUNAPPROVEDBYHQ', 'CANBEREJECTED', 'CANBERESTARTED', 'CANBEOPENED')


class InterviewStatus(sgqlc.types.Enum):
    __schema__ = headquarters_schema
    __choices__ = ('RESTORED', 'CREATED', 'SUPERVISORASSIGNED', 'INTERVIEWERASSIGNED', 'REJECTEDBYSUPERVISOR', 'READYFORINTERVIEW', 'SENTTOCAPI', 'RESTARTED', 'COMPLETED', 'APPROVEDBYSUPERVISOR', 'REJECTEDBYHEADQUARTERS', 'APPROVEDBYHEADQUARTERS', 'DELETED')


class Long(sgqlc.types.Scalar):
    __schema__ = headquarters_schema


class PaginationAmount(sgqlc.types.Scalar):
    __schema__ = headquarters_schema


class QuestionScope(sgqlc.types.Enum):
    __schema__ = headquarters_schema
    __choices__ = ('INTERVIEWER', 'SUPERVISOR', 'HEADQUARTER', 'HIDDEN')


class QuestionType(sgqlc.types.Enum):
    __schema__ = headquarters_schema
    __choices__ = ('SINGLEOPTION', 'MULTYOPTION', 'NUMERIC', 'DATETIME', 'GPSCOORDINATES', 'TEXT', 'TEXTLIST', 'QRBARCODE', 'MULTIMEDIA', 'AREA', 'AUDIO')


class SortOperationKind(sgqlc.types.Enum):
    __schema__ = headquarters_schema
    __choices__ = ('ASC', 'DESC')


String = sgqlc.types.String

class UserRoles(sgqlc.types.Enum):
    __schema__ = headquarters_schema
    __choices__ = ('ADMINISTRATOR', 'SUPERVISOR', 'INTERVIEWER', 'HEADQUARTER', 'OBSERVER', 'APIUSER')


class Uuid(sgqlc.types.Scalar):
    __schema__ = headquarters_schema



########################################################################
# Input Objects
########################################################################
class InterviewFilter(sgqlc.types.Input):
    __schema__ = headquarters_schema
    __field_names__ = ('and_', 'assignment_id', 'assignment_id_in', 'assignment_id_not', 'created_date_gt', 'created_date_gte', 'created_date_lt', 'created_date_lte', 'created_date_not_gt', 'created_date_not_gte', 'created_date_not_lt', 'created_date_not_lte', 'errors_count', 'errors_count_gt', 'identifying_questions_some', 'key', 'key_contains', 'key_in', 'key_starts_with', 'or_', 'questionnaire_id', 'questionnaire_variable', 'questionnaire_version', 'received_by_interviewer', 'responsible_name', 'responsible_name_lower_case', 'responsible_name_lower_case_in', 'responsible_name_lower_case_starts_with', 'responsible_name_in', 'responsible_name_starts_with', 'responsible_role', 'status', 'status_in', 'status_not', 'supervisor_name', 'supervisor_name_lower_case', 'supervisor_name_lower_case_in', 'supervisor_name_lower_case_starts_with', 'supervisor_name_in', 'supervisor_name_starts_with', 'update_date_gt', 'update_date_gte', 'update_date_lt', 'update_date_lte', 'update_date_not_gt', 'update_date_not_gte', 'update_date_not_lt', 'update_date_not_lte')
    and_ = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null('InterviewFilter')), graphql_name='AND')
    assignment_id = sgqlc.types.Field(Int, graphql_name='assignmentId')
    assignment_id_in = sgqlc.types.Field(sgqlc.types.list_of(Int), graphql_name='assignmentId_in')
    assignment_id_not = sgqlc.types.Field(Int, graphql_name='assignmentId_not')
    created_date_gt = sgqlc.types.Field(DateTime, graphql_name='createdDate_gt')
    created_date_gte = sgqlc.types.Field(DateTime, graphql_name='createdDate_gte')
    created_date_lt = sgqlc.types.Field(DateTime, graphql_name='createdDate_lt')
    created_date_lte = sgqlc.types.Field(DateTime, graphql_name='createdDate_lte')
    created_date_not_gt = sgqlc.types.Field(DateTime, graphql_name='createdDate_not_gt')
    created_date_not_gte = sgqlc.types.Field(DateTime, graphql_name='createdDate_not_gte')
    created_date_not_lt = sgqlc.types.Field(DateTime, graphql_name='createdDate_not_lt')
    created_date_not_lte = sgqlc.types.Field(DateTime, graphql_name='createdDate_not_lte')
    errors_count = sgqlc.types.Field(Int, graphql_name='errorsCount')
    errors_count_gt = sgqlc.types.Field(Int, graphql_name='errorsCount_gt')
    identifying_questions_some = sgqlc.types.Field('QuestionAnswerFilter', graphql_name='identifyingQuestions_some')
    key = sgqlc.types.Field(String, graphql_name='key')
    key_contains = sgqlc.types.Field(String, graphql_name='key_contains')
    key_in = sgqlc.types.Field(sgqlc.types.list_of(String), graphql_name='key_in')
    key_starts_with = sgqlc.types.Field(String, graphql_name='key_starts_with')
    or_ = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null('InterviewFilter')), graphql_name='OR')
    questionnaire_id = sgqlc.types.Field(Uuid, graphql_name='questionnaireId')
    questionnaire_variable = sgqlc.types.Field(String, graphql_name='questionnaireVariable')
    questionnaire_version = sgqlc.types.Field(Long, graphql_name='questionnaireVersion')
    received_by_interviewer = sgqlc.types.Field(Boolean, graphql_name='receivedByInterviewer')
    responsible_name = sgqlc.types.Field(String, graphql_name='responsibleName')
    responsible_name_lower_case = sgqlc.types.Field(String, graphql_name='responsibleNameLowerCase')
    responsible_name_lower_case_in = sgqlc.types.Field(sgqlc.types.list_of(String), graphql_name='responsibleNameLowerCase_in')
    responsible_name_lower_case_starts_with = sgqlc.types.Field(String, graphql_name='responsibleNameLowerCase_starts_with')
    responsible_name_in = sgqlc.types.Field(sgqlc.types.list_of(String), graphql_name='responsibleName_in')
    responsible_name_starts_with = sgqlc.types.Field(String, graphql_name='responsibleName_starts_with')
    responsible_role = sgqlc.types.Field(UserRoles, graphql_name='responsibleRole')
    status = sgqlc.types.Field(InterviewStatus, graphql_name='status')
    status_in = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null(InterviewStatus)), graphql_name='status_in')
    status_not = sgqlc.types.Field(InterviewStatus, graphql_name='status_not')
    supervisor_name = sgqlc.types.Field(String, graphql_name='supervisorName')
    supervisor_name_lower_case = sgqlc.types.Field(String, graphql_name='supervisorNameLowerCase')
    supervisor_name_lower_case_in = sgqlc.types.Field(sgqlc.types.list_of(String), graphql_name='supervisorNameLowerCase_in')
    supervisor_name_lower_case_starts_with = sgqlc.types.Field(String, graphql_name='supervisorNameLowerCase_starts_with')
    supervisor_name_in = sgqlc.types.Field(sgqlc.types.list_of(String), graphql_name='supervisorName_in')
    supervisor_name_starts_with = sgqlc.types.Field(String, graphql_name='supervisorName_starts_with')
    update_date_gt = sgqlc.types.Field(DateTime, graphql_name='updateDate_gt')
    update_date_gte = sgqlc.types.Field(DateTime, graphql_name='updateDate_gte')
    update_date_lt = sgqlc.types.Field(DateTime, graphql_name='updateDate_lt')
    update_date_lte = sgqlc.types.Field(DateTime, graphql_name='updateDate_lte')
    update_date_not_gt = sgqlc.types.Field(DateTime, graphql_name='updateDate_not_gt')
    update_date_not_gte = sgqlc.types.Field(DateTime, graphql_name='updateDate_not_gte')
    update_date_not_lt = sgqlc.types.Field(DateTime, graphql_name='updateDate_not_lt')
    update_date_not_lte = sgqlc.types.Field(DateTime, graphql_name='updateDate_not_lte')


class InterviewSort(sgqlc.types.Input):
    __schema__ = headquarters_schema
    __field_names__ = ('assignment_id', 'created_date', 'errors_count', 'id', 'key', 'questionnaire_id', 'questionnaire_version', 'received_by_interviewer', 'responsible_name', 'responsible_role', 'status', 'update_date')
    assignment_id = sgqlc.types.Field(SortOperationKind, graphql_name='assignmentId')
    created_date = sgqlc.types.Field(SortOperationKind, graphql_name='createdDate')
    errors_count = sgqlc.types.Field(SortOperationKind, graphql_name='errorsCount')
    id = sgqlc.types.Field(SortOperationKind, graphql_name='id')
    key = sgqlc.types.Field(SortOperationKind, graphql_name='key')
    questionnaire_id = sgqlc.types.Field(SortOperationKind, graphql_name='questionnaireId')
    questionnaire_version = sgqlc.types.Field(SortOperationKind, graphql_name='questionnaireVersion')
    received_by_interviewer = sgqlc.types.Field(SortOperationKind, graphql_name='receivedByInterviewer')
    responsible_name = sgqlc.types.Field(SortOperationKind, graphql_name='responsibleName')
    responsible_role = sgqlc.types.Field(SortOperationKind, graphql_name='responsibleRole')
    status = sgqlc.types.Field(SortOperationKind, graphql_name='status')
    update_date = sgqlc.types.Field(SortOperationKind, graphql_name='updateDate')


class MapsFilter(sgqlc.types.Input):
    __schema__ = headquarters_schema
    __field_names__ = ('and_', 'file_name', 'file_name_in', 'file_name_starts_with', 'import_date_gt', 'import_date_gte', 'import_date_lt', 'import_date_lte', 'import_date_not_gt', 'import_date_not_gte', 'import_date_not_lt', 'import_date_not_lte', 'or_', 'size_gt', 'size_gte', 'size_lt', 'size_lte', 'size_not_gt', 'size_not_gte', 'size_not_lt', 'size_not_lte', 'users_some')
    and_ = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null('MapsFilter')), graphql_name='AND')
    file_name = sgqlc.types.Field(String, graphql_name='fileName')
    file_name_in = sgqlc.types.Field(sgqlc.types.list_of(String), graphql_name='fileName_in')
    file_name_starts_with = sgqlc.types.Field(String, graphql_name='fileName_starts_with')
    import_date_gt = sgqlc.types.Field(DateTime, graphql_name='importDate_gt')
    import_date_gte = sgqlc.types.Field(DateTime, graphql_name='importDate_gte')
    import_date_lt = sgqlc.types.Field(DateTime, graphql_name='importDate_lt')
    import_date_lte = sgqlc.types.Field(DateTime, graphql_name='importDate_lte')
    import_date_not_gt = sgqlc.types.Field(DateTime, graphql_name='importDate_not_gt')
    import_date_not_gte = sgqlc.types.Field(DateTime, graphql_name='importDate_not_gte')
    import_date_not_lt = sgqlc.types.Field(DateTime, graphql_name='importDate_not_lt')
    import_date_not_lte = sgqlc.types.Field(DateTime, graphql_name='importDate_not_lte')
    or_ = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null('MapsFilter')), graphql_name='OR')
    size_gt = sgqlc.types.Field(Long, graphql_name='size_gt')
    size_gte = sgqlc.types.Field(Long, graphql_name='size_gte')
    size_lt = sgqlc.types.Field(Long, graphql_name='size_lt')
    size_lte = sgqlc.types.Field(Long, graphql_name='size_lte')
    size_not_gt = sgqlc.types.Field(Long, graphql_name='size_not_gt')
    size_not_gte = sgqlc.types.Field(Long, graphql_name='size_not_gte')
    size_not_lt = sgqlc.types.Field(Long, graphql_name='size_not_lt')
    size_not_lte = sgqlc.types.Field(Long, graphql_name='size_not_lte')
    users_some = sgqlc.types.Field('UserMapFilter', graphql_name='users_some')


class MapsSort(sgqlc.types.Input):
    __schema__ = headquarters_schema
    __field_names__ = ('file_name', 'import_date', 'size')
    file_name = sgqlc.types.Field(SortOperationKind, graphql_name='fileName')
    import_date = sgqlc.types.Field(SortOperationKind, graphql_name='importDate')
    size = sgqlc.types.Field(SortOperationKind, graphql_name='size')


class QuestionAnswerFilter(sgqlc.types.Input):
    __schema__ = headquarters_schema
    __field_names__ = ('and_', 'answer', 'answer_code', 'answer_code_in', 'answer_code_not', 'answer_code_not_in', 'answer_lower_case', 'answer_lower_case_not', 'answer_lower_case_not_starts_with', 'answer_lower_case_starts_with', 'answer_not', 'answer_not_starts_with', 'answer_starts_with', 'or_', 'question')
    and_ = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null('QuestionAnswerFilter')), graphql_name='AND')
    answer = sgqlc.types.Field(String, graphql_name='answer')
    answer_code = sgqlc.types.Field(Decimal, graphql_name='answerCode')
    answer_code_in = sgqlc.types.Field(sgqlc.types.list_of(Decimal), graphql_name='answerCode_in')
    answer_code_not = sgqlc.types.Field(Decimal, graphql_name='answerCode_not')
    answer_code_not_in = sgqlc.types.Field(sgqlc.types.list_of(Decimal), graphql_name='answerCode_not_in')
    answer_lower_case = sgqlc.types.Field(String, graphql_name='answerLowerCase')
    answer_lower_case_not = sgqlc.types.Field(String, graphql_name='answerLowerCase_not')
    answer_lower_case_not_starts_with = sgqlc.types.Field(String, graphql_name='answerLowerCase_not_starts_with')
    answer_lower_case_starts_with = sgqlc.types.Field(String, graphql_name='answerLowerCase_starts_with')
    answer_not = sgqlc.types.Field(String, graphql_name='answer_not')
    answer_not_starts_with = sgqlc.types.Field(String, graphql_name='answer_not_starts_with')
    answer_starts_with = sgqlc.types.Field(String, graphql_name='answer_starts_with')
    or_ = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null('QuestionAnswerFilter')), graphql_name='OR')
    question = sgqlc.types.Field('QuestionFilter', graphql_name='question')


class QuestionFilter(sgqlc.types.Input):
    __schema__ = headquarters_schema
    __field_names__ = ('and_', 'identifying', 'or_', 'question_text', 'scope', 'variable')
    and_ = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null('QuestionFilter')), graphql_name='AND')
    identifying = sgqlc.types.Field(Boolean, graphql_name='identifying')
    or_ = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null('QuestionFilter')), graphql_name='OR')
    question_text = sgqlc.types.Field(String, graphql_name='questionText')
    scope = sgqlc.types.Field(QuestionScope, graphql_name='scope')
    variable = sgqlc.types.Field(String, graphql_name='variable')


class UserMapFilter(sgqlc.types.Input):
    __schema__ = headquarters_schema
    __field_names__ = ('and_', 'or_', 'user_name', 'user_name_not', 'user_name_not_starts_with', 'user_name_starts_with')
    and_ = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null('UserMapFilter')), graphql_name='AND')
    or_ = sgqlc.types.Field(sgqlc.types.list_of(sgqlc.types.non_null('UserMapFilter')), graphql_name='OR')
    user_name = sgqlc.types.Field(String, graphql_name='userName')
    user_name_not = sgqlc.types.Field(String, graphql_name='userName_not')
    user_name_not_starts_with = sgqlc.types.Field(String, graphql_name='userName_not_starts_with')
    user_name_starts_with = sgqlc.types.Field(String, graphql_name='userName_starts_with')



########################################################################
# Output Objects and Interfaces
########################################################################
class CategoricalOption(sgqlc.types.Type):
    __schema__ = headquarters_schema
    __field_names__ = ('parent_value', 'title', 'value')
    parent_value = sgqlc.types.Field(Int, graphql_name='parentValue')
    title = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='title')
    value = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='value')


class HeadquartersMutations(sgqlc.types.Type):
    __schema__ = headquarters_schema
    __field_names__ = ('add_user_to_map', 'delete_map', 'delete_user_from_map')
    add_user_to_map = sgqlc.types.Field('Map', graphql_name='addUserToMap', args=sgqlc.types.ArgDict((
        ('file_name', sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name='fileName', default=None)),
        ('user_name', sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name='userName', default=None)),
))
    )
    delete_map = sgqlc.types.Field('Map', graphql_name='deleteMap', args=sgqlc.types.ArgDict((
        ('file_name', sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name='fileName', default=None)),
))
    )
    delete_user_from_map = sgqlc.types.Field('Map', graphql_name='deleteUserFromMap', args=sgqlc.types.ArgDict((
        ('file_name', sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name='fileName', default=None)),
        ('user_name', sgqlc.types.Arg(sgqlc.types.non_null(String), graphql_name='userName', default=None)),
))
    )


class HeadquartersQuery(sgqlc.types.Type):
    __schema__ = headquarters_schema
    __field_names__ = ('interviews', 'maps', 'questionnaires', 'questions', 'viewer')
    interviews = sgqlc.types.Field('IPagedConnectionOfInterview', graphql_name='interviews', args=sgqlc.types.ArgDict((
        ('order_by', sgqlc.types.Arg(InterviewSort, graphql_name='order_by', default=None)),
        ('skip', sgqlc.types.Arg(Int, graphql_name='skip', default=None)),
        ('take', sgqlc.types.Arg(Int, graphql_name='take', default=None)),
        ('where', sgqlc.types.Arg(InterviewFilter, graphql_name='where', default=None)),
))
    )
    maps = sgqlc.types.Field('IPagedConnectionOfMap', graphql_name='maps', args=sgqlc.types.ArgDict((
        ('order_by', sgqlc.types.Arg(MapsSort, graphql_name='order_by', default=None)),
        ('skip', sgqlc.types.Arg(Int, graphql_name='skip', default=None)),
        ('take', sgqlc.types.Arg(Int, graphql_name='take', default=None)),
        ('where', sgqlc.types.Arg(MapsFilter, graphql_name='where', default=None)),
))
    )
    questionnaires = sgqlc.types.Field('IPagedConnectionOfQuestionnaire', graphql_name='questionnaires', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(Uuid, graphql_name='id', default=None)),
        ('skip', sgqlc.types.Arg(Int, graphql_name='skip', default=None)),
        ('take', sgqlc.types.Arg(Int, graphql_name='take', default=None)),
        ('version', sgqlc.types.Arg(Long, graphql_name='version', default=None)),
))
    )
    questions = sgqlc.types.Field(sgqlc.types.list_of('Question'), graphql_name='questions', args=sgqlc.types.ArgDict((
        ('id', sgqlc.types.Arg(sgqlc.types.non_null(Uuid), graphql_name='id', default=None)),
        ('language', sgqlc.types.Arg(String, graphql_name='language', default=None)),
        ('version', sgqlc.types.Arg(sgqlc.types.non_null(Long), graphql_name='version', default=None)),
        ('where', sgqlc.types.Arg(QuestionFilter, graphql_name='where', default=None)),
))
    )
    viewer = sgqlc.types.Field('User', graphql_name='viewer')


class IPagedConnectionOfInterview(sgqlc.types.Type):
    __schema__ = headquarters_schema
    __field_names__ = ('filtered_count', 'nodes', 'total_count')
    filtered_count = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='filteredCount')
    nodes = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of('Interview')), graphql_name='nodes')
    total_count = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='totalCount')


class IPagedConnectionOfMap(sgqlc.types.Type):
    __schema__ = headquarters_schema
    __field_names__ = ('filtered_count', 'nodes', 'total_count')
    filtered_count = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='filteredCount')
    nodes = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of('Map')), graphql_name='nodes')
    total_count = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='totalCount')


class IPagedConnectionOfQuestionnaire(sgqlc.types.Type):
    __schema__ = headquarters_schema
    __field_names__ = ('filtered_count', 'nodes', 'total_count')
    filtered_count = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='filteredCount')
    nodes = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of('Questionnaire')), graphql_name='nodes')
    total_count = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='totalCount')


class Interview(sgqlc.types.Type):
    __schema__ = headquarters_schema
    __field_names__ = ('action_flags', 'assignment_id', 'created_date', 'errors_count', 'id', 'identifying_questions', 'key', 'questionnaire_id', 'questionnaire_variable', 'questionnaire_version', 'received_by_interviewer', 'responsible_id', 'responsible_name', 'responsible_name_lower_case', 'responsible_role', 'status', 'supervisor_name', 'supervisor_name_lower_case', 'update_date', 'was_completed')
    action_flags = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(InterviewActionFlags))), graphql_name='actionFlags')
    assignment_id = sgqlc.types.Field(Int, graphql_name='assignmentId')
    created_date = sgqlc.types.Field(sgqlc.types.non_null(DateTime), graphql_name='createdDate')
    errors_count = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='errorsCount')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    identifying_questions = sgqlc.types.Field(sgqlc.types.list_of('QuestionAnswer'), graphql_name='identifyingQuestions')
    key = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='key')
    questionnaire_id = sgqlc.types.Field(sgqlc.types.non_null(Uuid), graphql_name='questionnaireId')
    questionnaire_variable = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='questionnaireVariable')
    questionnaire_version = sgqlc.types.Field(sgqlc.types.non_null(Long), graphql_name='questionnaireVersion')
    received_by_interviewer = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='receivedByInterviewer')
    responsible_id = sgqlc.types.Field(sgqlc.types.non_null(Uuid), graphql_name='responsibleId')
    responsible_name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='responsibleName')
    responsible_name_lower_case = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='responsibleNameLowerCase')
    responsible_role = sgqlc.types.Field(sgqlc.types.non_null(UserRoles), graphql_name='responsibleRole')
    status = sgqlc.types.Field(sgqlc.types.non_null(InterviewStatus), graphql_name='status')
    supervisor_name = sgqlc.types.Field(String, graphql_name='supervisorName')
    supervisor_name_lower_case = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='supervisorNameLowerCase')
    update_date = sgqlc.types.Field(sgqlc.types.non_null(DateTime), graphql_name='updateDate')
    was_completed = sgqlc.types.Field(sgqlc.types.non_null(Boolean), graphql_name='wasCompleted')


class Map(sgqlc.types.Type):
    __schema__ = headquarters_schema
    __field_names__ = ('file_name', 'import_date', 'max_scale', 'min_scale', 'size', 'users', 'wkid', 'x_max_val', 'x_min_val', 'y_max_val', 'y_min_val')
    file_name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='fileName')
    import_date = sgqlc.types.Field(DateTime, graphql_name='importDate')
    max_scale = sgqlc.types.Field(sgqlc.types.non_null(Float), graphql_name='maxScale')
    min_scale = sgqlc.types.Field(sgqlc.types.non_null(Float), graphql_name='minScale')
    size = sgqlc.types.Field(sgqlc.types.non_null(Long), graphql_name='size')
    users = sgqlc.types.Field(sgqlc.types.list_of('UserMap'), graphql_name='users')
    wkid = sgqlc.types.Field(sgqlc.types.non_null(Int), graphql_name='wkid')
    x_max_val = sgqlc.types.Field(sgqlc.types.non_null(Float), graphql_name='xMaxVal')
    x_min_val = sgqlc.types.Field(sgqlc.types.non_null(Float), graphql_name='xMinVal')
    y_max_val = sgqlc.types.Field(sgqlc.types.non_null(Float), graphql_name='yMaxVal')
    y_min_val = sgqlc.types.Field(sgqlc.types.non_null(Float), graphql_name='yMinVal')


class Question(sgqlc.types.Type):
    __schema__ = headquarters_schema
    __field_names__ = ('identifying', 'label', 'options', 'question_text', 'scope', 'type', 'variable')
    identifying = sgqlc.types.Field(Boolean, graphql_name='identifying')
    label = sgqlc.types.Field(String, graphql_name='label')
    options = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(CategoricalOption))), graphql_name='options')
    question_text = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='questionText')
    scope = sgqlc.types.Field(QuestionScope, graphql_name='scope')
    type = sgqlc.types.Field(sgqlc.types.non_null(QuestionType), graphql_name='type')
    variable = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='variable')


class QuestionAnswer(sgqlc.types.Type):
    __schema__ = headquarters_schema
    __field_names__ = ('answer', 'answer_lower_case', 'answer_value', 'question')
    answer = sgqlc.types.Field(String, graphql_name='answer')
    answer_lower_case = sgqlc.types.Field(String, graphql_name='answerLowerCase')
    answer_value = sgqlc.types.Field(Int, graphql_name='answerValue')
    question = sgqlc.types.Field(sgqlc.types.non_null(Question), graphql_name='question')


class Questionnaire(sgqlc.types.Type):
    __schema__ = headquarters_schema
    __field_names__ = ('default_language_name', 'id', 'questionnaire_id', 'title', 'translations', 'variable', 'version')
    default_language_name = sgqlc.types.Field(String, graphql_name='defaultLanguageName')
    id = sgqlc.types.Field(ID, graphql_name='id')
    questionnaire_id = sgqlc.types.Field(sgqlc.types.non_null(Uuid), graphql_name='questionnaireId')
    title = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='title')
    translations = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null('Translation'))), graphql_name='translations')
    variable = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='variable')
    version = sgqlc.types.Field(sgqlc.types.non_null(Long), graphql_name='version')


class Translation(sgqlc.types.Type):
    __schema__ = headquarters_schema
    __field_names__ = ('id', 'name')
    id = sgqlc.types.Field(sgqlc.types.non_null(Uuid), graphql_name='id')
    name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='name')


class User(sgqlc.types.Type):
    __schema__ = headquarters_schema
    __field_names__ = ('id', 'roles', 'user_name')
    id = sgqlc.types.Field(sgqlc.types.non_null(ID), graphql_name='id')
    roles = sgqlc.types.Field(sgqlc.types.non_null(sgqlc.types.list_of(sgqlc.types.non_null(UserRoles))), graphql_name='roles')
    user_name = sgqlc.types.Field(sgqlc.types.non_null(String), graphql_name='userName')


class UserMap(sgqlc.types.Type):
    __schema__ = headquarters_schema
    __field_names__ = ('user_name',)
    user_name = sgqlc.types.Field(String, graphql_name='userName')



########################################################################
# Unions
########################################################################

########################################################################
# Schema Entry Points
########################################################################
headquarters_schema.query_type = HeadquartersQuery
headquarters_schema.mutation_type = HeadquartersMutations
headquarters_schema.subscription_type = None




from .base import HQBase


class SettingsApi(HQBase):
    _apiprefix = "/api/v1/settings"

    def get_globalnotice(self):
        """GET /api/v1/settings/globalnotice"""

        path = self.url + '/globalnotice'
        r = self._make_call('get', path)
        return str(r['Message'] or '')

    def set_globalnotice(self, message):
        """PUT /api/v1/settings/globalnotice"""

        path = self.url + '/globalnotice'
        self._make_call('put', path, json={'message': message})

    def remove_globalnotice(self):
        """DELETE /api/v1/settings/globalnotice"""

        path = self.url + '/globalnotice'
        self._make_call('delete', path)



from .designer import (
    collect_expressions,
    create_questionnaire,
    export_questionnaire,
    import_questionnaire,
    import_questionnaire_json,
)


__all__ = ["collect_expressions", "create_questionnaire",
           "export_questionnaire", "import_questionnaire", "import_questionnaire_json", ]



from html import escape
from typing import Generator

from sgqlc.operation import Operation

from .base import HQBase
from .exceptions import GraphQLError
from .headquarters_schema import Interview, InterviewFilter, headquarters_schema
from .models import InterviewAnswers
from .utils import fix_qid


class InterviewsApi(HQBase):
    _apiprefix = "/api/v1/interviews"

    @fix_qid(expects={'questionnaire_id': 'hex'})
    def get_list(self, fields: list = [], **kwargs) -> Generator[Interview, None, None]:
        where = InterviewFilter(**kwargs)
        take = 20
        skip = 0
        filtered_count = 21
        if not fields:
            fields = [
                'id',
                'questionnaire_id',
                'questionnaire_version',
                'assignment_id',
                'responsible_id',
                'errors_count',
                'status',
            ]
        while skip < filtered_count:
            op = Operation(headquarters_schema.HeadquartersQuery)
            q = op.interviews(take=take, skip=skip, where=where)
            q.__fields__('filtered_count')
            q.nodes.__fields__(*fields)
            print(op)
            cont = self.endpoint(op)
            errors = cont.get('errors')
            if errors:
                raise GraphQLError(errors[0]['message'])
            res = (op + cont).interviews

            filtered_count = res.filtered_count
            yield from res.nodes
            skip += take

    def get_info(self, interview_id: str) -> list:
        path = self.url + '/{}'.format(interview_id)
        ret = self._make_call('get', path)
        if "Answers" in ret:
            return InterviewAnswers(ret)

    def delete(self, interviewid, comment=''):
        return self._change_status(action='delete', interviewid=interviewid, comment=comment)

    def approve(self, interviewid, comment=''):
        return self._change_status(action='approve', interviewid=interviewid, comment=comment)

    def assign(self, interviewid, responsibleid, responsiblename=''):
        return self._reassign(action='assign', interviewid=interviewid,
                              responsibleid=responsibleid, responsiblename=responsiblename)

    def assign_supervisor(self, interviewid, responsibleid, responsiblename=''):
        return self._reassign(action='assignsupervisor', interviewid=interviewid,
                              responsibleid=responsibleid, responsiblename=responsiblename)

    @fix_qid(expects={'interview_id': 'string', 'question_id': 'hex'})
    def comment(self, interview_id, comment, question_id: str = None, variable: str = None, roster_vector: list = []):
        params = {'comment': escape(comment)}
        if roster_vector:
            params['rosterVector'] = roster_vector

        if variable:
            path = self.url + '/{}/comment-by-variable/{}'.format(interview_id, variable)
        else:
            if question_id:
                path = self.url + '/{}/comment/{}'.format(interview_id, question_id)
            else:
                raise TypeError("comment() either 'variable' or 'question_id' argument is required")

        self._make_call('post', path, params=params)

    @fix_qid(expects={'interview_id': 'string'})
    def history(self, interview_id):
        path = self.url + '/{}/history'.format(interview_id)
        return self._make_call('get', path)

    def hqapprove(self, interviewid, comment=''):
        return self._change_status(action='hqapprove', interviewid=interviewid, comment=comment)

    def hqreject(self, interviewid, comment=''):
        return self._change_status(action='hqreject', interviewid=interviewid, comment=comment)

    def hqunapprove(self, interviewid, comment=''):
        return self._change_status(action='hqunapprove', interviewid=interviewid, comment=comment)

    def reject(self, interviewid, comment=''):
        return self._change_status(action='reject', interviewid=interviewid, comment=comment)

    def stats(self, interviewid):
        pass

    def _change_status(self, interviewid, action, comment=''):
        path = self.url + '/{}/{}'.format(interviewid, action)
        return self._make_call('patch', path, params={'comment': escape(comment)})

    def _reassign(self, interviewid, action, responsibleid, responsiblename=''):
        path = self.url + '/{}/{}'.format(interviewid, action)
        payload = {
            'ResponsibleId': responsibleid,
            'ResponsibleName': responsiblename
        }
        _ = self._make_call('patch', path, json=payload)
        return True



import uuid
from functools import wraps


def fix_qid(expects: dict = {'questionnaire_id': 'hex'}):
    def wrapper_outer(func):
        @wraps(func)
        def wrapper_inner(*args, **kwargs):
            for param_name in expects.keys():
                if param_name in kwargs:
                    if expects[param_name] == 'hex':
                        kwargs[param_name] = uuid.UUID(kwargs[param_name]).hex
                    elif expects[param_name] == 'string':
                        kwargs[param_name] = str(uuid.UUID(kwargs[param_name]))
                    else:
                        raise ValueError('expects should be either hex or string')
            return func(*args, **kwargs)
        return wrapper_inner
    return wrapper_outer


def to_hex(q_id):
    return uuid.UUID(q_id).hex


def to_qidentity(q_id, q_version):
    return "{}${}".format(to_hex(q_id), q_version)


def parse_qidentity(q_identity):
    if type(q_identity) is tuple:
        (q_id, q_version) = q_identity
    else:
        qq = q_identity.split("$")
        q_id = qq[0]
        q_version = qq[1]

    return to_qidentity(q_id, q_version)



from .base import HQBase


class UsersApi(HQBase):
    _apiprefix = "/api/v1"

    def get_info(self, id):
        path = self._url_users + '/{}'.format(id)
        return self._make_call('get', path)

    def get_actions_log(self, id):
        path = self._url_interviewers + '/{}/actions-log'.format(id)
        return self._make_call('get', path)

    def list_supervisors(self):
        path = self._url_supervisors
        return self._list_users(path)

    def list_interviewers(self, id):
        path = self._url_supervisors + '/{}/interviewers'.format(id)
        return self._list_users(path)

    def unarchive(self, id):
        path = self._url_users + '/{}/unarchive'.format(id)
        return self._make_call('patch', path)

    def archive(self, id):
        path = self._url_users + '/{}/archive'.format(id)
        return self._make_call('patch', path)

    def _list_users(self, path):
        page_size = 10
        page = 1
        total_count = 11
        params = {
            'offset': page,
            'limit': page_size
        }
        while page * page_size < total_count:
            params['offset'] = page
            r = self._make_call('get', path, params=params)
            total_count = r['TotalCount']
            if total_count > 0:
                yield from r['Users']
            else:
                yield from ()
            page += 1

    @property
    def _url_users(self):
        return self.url + '/users'

    @property
    def _url_supervisors(self):
        return self.url + '/supervisors'

    @property
    def _url_interviewers(self):
        return self.url + '/interviewers'



from . import designer
from .__about__ import __version__
from .assignments import AssignmentsApi
from .export import ExportApi
from .headquarters import Client
from .interviews import InterviewsApi
from .questionnaires import QuestionnairesApi
from .settings import SettingsApi
from .users import UsersApi


__all__ = ["designer", "__version__", "AssignmentsApi", "ExportApi",
           "Client", "InterviewsApi", "QuestionnairesApi", "SettingsApi", "UsersApi", ]



import copy
import hashlib
import imghdr
import json
import os
import shutil
import tempfile
import uuid
import zipfile
from datetime import datetime

from .constants import CLASSTYPE_LOOKUPTABLE, CLASSTYPE_MACRO, QUESTION_TYPES


def export_questionnaire(obj, outfolder):
    obj.PublicKey = str(uuid.uuid4())
    tmppath = tempfile.gettempdir()
    subfolder = obj.Title + " (" + obj.PublicKey + ")"
    path = os.path.join(tmppath, subfolder)
    filename = obj.Title + ".json"
    os.mkdir(path)
    tmpfile = os.path.join(path, filename)
    with open(tmpfile, "w") as outfile:
        json.dump(obj, outfile, cls=AutoJSONEncoder, indent=4, sort_keys=True)

    obj.export_lookuptables(path)
    obj.export_attachments(path)
    obj.export_translations(path)

    zipfile = os.path.join(outfolder, obj.Title)
    shutil.make_archive(zipfile, "zip", tmppath, subfolder)
    shutil.rmtree(path)


def import_questionnaire_json(jsonstring):
    return json.loads(jsonstring, object_hook=decode_object)


def import_questionnaire(archive):
    tmpdir = os.path.join(tempfile.gettempdir(), str(uuid.uuid4()))
    filename = ""
    with zipfile.ZipFile(archive) as zf:
        zf.extractall(tmpdir)

    for root, dirs, files in os.walk(tmpdir):
        for file in files:
            if ".json" in file:
                filename = os.path.join(root, file)
                break

    lookupfolder = os.path.join(os.path.dirname(filename), "Lookup Tables")
    attachmentfolder = os.path.join(os.path.dirname(filename), "Attachments")
    translationfolder = os.path.join(os.path.dirname(filename), "Translations")

    with open(filename, "r") as fp:
        ret = json.load(fp, object_hook=decode_object)

    ret.import_lookuptables(lookupfolder)
    ret.import_attachments(attachmentfolder)
    ret.import_translations(translationfolder)

    shutil.rmtree(tmpdir)
    return ret


def clean_string(candidate):
    return candidate.replace("\n", " ").replace("\r", "").strip()


class MacrosCache:
    def __init__(self):
        self._list = {}

    def add(self, name, value):
        self._list[name] = value

    def evaluate(self, candidate):
        ret = candidate
        for name, value in self._list.items():
            ret = ret.replace("$" + name, value)
        return ret


def collect_expressions(obj, id="", evaluate_macros=False, macros_cache=None):  # noqa: C901
    ret = []
    if id == "":
        if hasattr(obj, "PublicKey"):
            id = obj.PublicKey
        else:
            try:
                show_structure(obj)
            except Exception:
                print(obj)
        if evaluate_macros and macros_cache is None:
            macros_cache = MacrosCache()

    if hasattr(obj, "Macros"):
        for key in obj.Macros:
            if key != "$type":
                m = obj.Macros[key]
                if "Content" in m:
                    cont = clean_string(m["Content"])
                    if cont:
                        name = m["Name"] if "Name" in m else "no_name"
                        if evaluate_macros:
                            macros_cache.add(name, cont)
                        else:
                            ret.append((id, "Macro", "", name, cont))

    if hasattr(obj, "StataExportCaption"):
        variablename = obj.StataExportCaption
    else:
        variablename = obj.Name if hasattr(obj, "Name") else ""
    if hasattr(obj, "ConditionExpression"):
        cont = clean_string(obj.ConditionExpression)
        if cont:
            if evaluate_macros:
                cont = macros_cache.evaluate(cont)
            ret.append((id, "Enablement", obj.PublicKey, variablename, cont))

    if hasattr(obj, "ValidationConditions"):
        for v in obj.ValidationConditions:
            if "Expression" in v:
                cont = clean_string(v["Expression"])
                if cont:
                    if evaluate_macros:
                        cont = macros_cache.evaluate(cont)
                    ret.append((id, "Validation", obj.PublicKey, variablename, cont))

    if hasattr(obj, "Expression"):  # variables
        cont = clean_string(obj.Expression)
        if cont:
            if evaluate_macros:
                cont = macros_cache.evaluate(cont)
            ret.append((id, "Variable", "", obj.Name, cont))

    if hasattr(obj, "Properties"):
        cont = ""
        if type(obj.Properties) is dict:
            if "OptionsFilterExpression" in obj.Properties:
                cont = clean_string(obj.Properties["OptionsFilterExpression"])
        else:
            if hasattr(obj.Properties, "OptionsFilterExpression"):
                cont = clean_string(obj.Properties.OptionsFilterExpression)

        if cont:
            if evaluate_macros:
                cont = macros_cache.evaluate(cont)
            ret.append((id, "Filter", obj.PublicKey, variablename, cont))

    if hasattr(obj, "Children"):
        for ch in obj.Children:
            a = collect_expressions(
                ch, id, evaluate_macros=evaluate_macros, macros_cache=macros_cache
            )
            if a:
                ret += a

    return ret


class AutoJSONEncoder(json.JSONEncoder):
    def default(self, obj):
        try:
            return obj._json()
        except AttributeError:
            return json.JSONEncoder.default(self, obj)


def decode_object(o):

    if "$type" in o:
        typevalue = o["$type"]
        del o["$type"]
    else:
        # some questionnaire documents don't have the proper type
        questionnaire_keys = [
            "CreatedBy",
            "CreationDate",
            "LastEventSequence",
            "SharedPersons",
            "UsesCSharp",
        ]
        if any(i in o for i in questionnaire_keys):
            typevalue = "QuestionnaireDocument"
        else:
            # extra info that we don't need
            extra_keys = [
                "ExpressionsPlayOrder",
                "DependencyGraph",
                "ValidationDependencyGraph",
            ]
            if all(i not in o for i in extra_keys):
                return o

    if typevalue in CLASS_DICT:
        if typevalue in [CLASSTYPE_LOOKUPTABLE, CLASSTYPE_MACRO]:
            o["$type"] = typevalue
            ret = create_object("MyDict", dict_type=typevalue, dict_obj=o)
        else:
            o["_Type"] = typevalue
            ret = create_object(typevalue, dict_obj=o)
    else:
        print(typevalue)
        print(o)
        raise TypeError

    return ret


def create_object(objtype, *args, **kwargs):
    if objtype in CLASS_DICT:
        obj = CLASS_DICT[objtype]()

        if "dict_obj" in kwargs:
            if objtype == "MyDict":
                obj = kwargs["dict_obj"]
            else:
                obj.import_dict(kwargs["dict_obj"])
        else:
            obj.create(*args, **kwargs)

        return obj
    else:
        raise AttributeError


def create_questionnaire(title, defaultsection=False):
    return create_object(
        "QuestionnaireDocument", title=title, defaultsection=defaultsection
    )


def question(questiontype=7, **kwargs):
    if questiontype in [0, 3, 4, 7]:
        q = create_object(
            QUESTION_TYPES[questiontype], questiontype=questiontype, **kwargs
        )
    else:
        q = create_object("Question", questiontype=questiontype, **kwargs)

    return q


def answer(value, text):
    return create_object("Answer", value, text)


def show_structure(obj, indent=0):
    print("-" * indent, obj._json())
    if hasattr(obj, "Children"):
        for item in obj.Children:
            show_structure(item, indent + 1)


class BaseClass(object):
    def import_dict(self, o):
        self.__dict__ = {**self.__dict__, **o}

    def _json(self):
        type = self.__dict__.get("_Type", self.__class__.__name__)
        ret = {"$type": type}
        for key, value in self.__dict__.items():
            if not key.startswith("_"):
                ret[key] = value
        return ret


class Questionnaire(BaseClass):
    def __init__(self):
        self._Type = "QuestionnaireDocument"
        self.PublicKey = ""
        self.Title = ""
        self.CreationDate = datetime.utcnow().isoformat()
        self.CreatedBy = "00000000-0000-0000-0000-000000000000"
        self.HideIfDisabled = False
        self.Children = []
        self.Macros = create_object("MyDict", dict_type=CLASSTYPE_MACRO)
        self.Attachments = []
        self.LookupTables = create_object("MyDict", dict_type=CLASSTYPE_LOOKUPTABLE)
        self.Translations = []
        self.SharedPersons = []

    def create(self, title, defaultsection=True):
        self.Title = title
        if defaultsection:
            self.Children.append(create_object("Group", self))

    def add_section(self, title):
        self.Children.append(create_object("Group", self, title))
        return self.Children[-1]

    def add_macro(self, **kwargs):
        item = {str(uuid.uuid4()): create_object("Macro", **kwargs)}
        self.Macros.update(item)

    def add_lookuptable(self, **kwargs):
        item = {str(uuid.uuid4()): create_object("LookupTable", **kwargs)}
        self.LookupTables.update(item)

    def add_attachment(self, **kwargs):
        self.Attachments.append(create_object("Attachment", **kwargs))

    def add_translation(self, **kwargs):
        self.Translations.append(create_object("Translation", **kwargs))

    def export_translationtemplate(self, outfolder):
        pass

    def export_lookuptables(self, outfolder):
        if len(self.LookupTables.keys()) > 1:
            ff = os.path.join(outfolder, "Lookup Tables")
            os.mkdir(ff)
            for key in self.LookupTables:
                if key != "$type":
                    filename = key.replace("-", "") + ".txt"
                    filename = os.path.join(ff, filename)
                    with open(filename, "w") as f:
                        f.writelines(self.LookupTables[key]._FileContent)

    def import_lookuptables(self, infolder):
        for key in self.LookupTables:
            if key != "$type":
                filename = os.path.join(infolder, key.replace("-", "") + ".txt")
                if type(self.LookupTables[key]) == "LookupTable":
                    self.LookupTables[key].import_content(filename)

    def export_attachments(self, outfolder):
        if len(self.Attachments):
            ff = os.path.join(outfolder, "Attachments")
            os.mkdir(ff)
            for item in self.Attachments:
                itemfolder = os.path.join(ff, item.AttachmentId.replace("-", ""))
                os.mkdir(itemfolder)
                filename = os.path.join(itemfolder, item._FileName)
                with open(filename, "wb") as f:
                    f.write(item._FileContent)

                filename = os.path.join(itemfolder, "Content-Type.txt")
                with open(filename, "w") as f:
                    f.writelines("image/" + item._ImageType)

    def import_attachments(self, infolder):
        for item in self.Attachments:
            itemfolder = os.path.join(infolder, item.AttachmentId.replace("-", ""))
            for file in os.listdir(itemfolder):
                if file != "Content-Type.txt":
                    item.import_content(os.path.join(itemfolder, file))

    def export_translations(self, outfolder):
        if len(self.Translations):
            ff = os.path.join(outfolder, "Translations")
            os.mkdir(ff)
            for item in self.Translations:
                filename = os.path.join(ff, item.Id.replace("-", "") + ".xlsx")
                with open(filename, "wb") as f:
                    f.write(item._FileContent)

    def import_translations(self, infolder):
        for item in self.Translations:
            filename = os.path.join(infolder, item.Id.replace("-", "") + ".xlsx")
            item.import_content(filename)

    def variables(self, obj=None):
        if not obj:
            obj = self
        ret = {}
        if obj._Type in ["Group", "QuestionnaireDocument"]:
            for ch in obj.Children:
                chret = self.variables(ch)
                if chret:
                    ret.update(chret)
        elif obj._Type not in ["StaticText"]:
            ret = {obj.VariableName: obj}
        return ret


class Group(BaseClass):
    def create(
        self,
        parent,
        title="New Section",
        condition="",
        hideifdisabled=False,
        description="",
    ):
        self.__parent__ = parent
        self.PublicKey = str(uuid.uuid4())
        self.Title = title
        self.ConditionExpression = condition
        self.HideIfDisabled = hideifdisabled
        self.Description = description
        self.IsRoster = False
        self.RosterSizeSource = 0
        self.FixedRosterTitles = []
        self.Children = []

    def add_question(self, questiontype=7, **kwargs):
        return self.add_item(question(questiontype, **kwargs))

    def add_subsection(self, title):
        self.Children.append(create_object("Group", self, title))
        return self.Children[-1]

    def add_roster(self):
        pass

    def add_statictext(self, *args, **kwargs):
        self.add_item(create_object("StaticText", *args, **kwargs))

    def add_variable(self, *args, **kwargs):
        self.add_item(create_object("Variable", *args, **kwargs))

    def add_item(self, obj_q):
        ref = copy.deepcopy(obj_q)
        self.Children.append(ref)
        return ref


class FixedRosterTitle(object):
    def __init__(self, value=1, title="First Title", object=None):
        if object:
            self.__dict__ = {**self.__dict__, **object}
        else:
            self.Value = value
            self.Title = title


class QuestionFactory(BaseClass):
    def create(
        self,
        questiontext,
        questiontype,
        variablename="",
        variablelabel="",
        instructions="",
        questionscope=0,
        hideifdisabled=False,
    ):
        self.PublicKey = str(uuid.uuid4())
        self.HideIfDisabled = hideifdisabled
        self.Instructions = instructions
        self.QuestionScope = questionscope
        self.QuestionText = questiontext
        self.QuestionType = questiontype
        self.StataExportCaption = variablename
        self.VariableLabel = variablelabel
        self.Properties = create_object("QuestionProperties")
        self.ValidationConditions = []
        self._Type = QUESTION_TYPES[self.QuestionType]

    def add_validation(self, expression="", message="", v_obj=None):
        if v_obj is None:
            v_obj = create_object("ValidationCondition", expression, message)
        self.ValidationConditions.append(v_obj)


class TextQuestion(QuestionFactory):
    def create(self, mask="", **kwargs):
        self.Mask = mask
        super(TextQuestion, self).create(**kwargs)


class NumericQuestion(QuestionFactory):
    def create(self, isinteger=False, **kwargs):
        self.IsInteger = isinteger
        super(NumericQuestion, self).create(**kwargs)


class SingleQuestion(QuestionFactory):
    def create(self, combobox=False, cascading_parent=None, **kwargs):
        self.IsFilteredCombobox = combobox
        if cascading_parent:
            self.CascadeFromQuestionId = cascading_parent
        self.Answers = []
        super(SingleQuestion, self).create(**kwargs)

    def add_option(self, *args, **kwargs):
        self.Answers.append(answer(*args, **kwargs))


class MultyOptionsQuestion(SingleQuestion):
    def create(self, ordered=False, yesno=False, **kwargs):
        self.AreAnswersOrdered = ordered
        self.YesNoView = yesno
        super(MultyOptionsQuestion, self).create(**kwargs)


class TextListQuestion(QuestionFactory):
    def __init__(self, **kwargs):
        super(TextListQuestion, self).__init__(**kwargs)


class ValidationCondition(BaseClass):
    def create(self, expression, message):
        self.Expression = expression
        self.Message = message


class QuestionProperties(BaseClass):
    def create(self):
        self.HideInstructions = False
        self.UseFormatting = False
        self.OptionsFilterExpression = ""


class AnswerFactory(BaseClass):
    def create(self, value, text):
        self._Type = "Answer"
        self.PublicKey = str(uuid.uuid4())
        self.AnswerText = text
        self.AnswerValue = value


class StaticText(BaseClass):
    def create(self, text, hideifdisabled=False):
        self.PublicKey = str(uuid.uuid4())
        self.Text = text
        self.AttachmentName = ""
        self.HideIfDisabled = hideifdisabled
        self.ValidationConditions = []


class Variable(BaseClass):
    def __init__(self):
        self.PublicKey = str(uuid.uuid4())
        self.Name = ""
        self.Label = ""
        self.Type = 3
        self.Expression = ""
        self.VariableName = ""

    def create(self, name="", type=3, expression="", label=""):
        self.Name = name
        self.Label = label
        self.Type = type
        self.Expression = expression
        self.VariableName = name


class MyDict(dict):
    def create(self, dict_type):
        self.update({"$type": dict_type})


class Macro(BaseClass):
    def create(self, name, content, description=""):
        self.Name = name
        self.Content = content
        self.Description = description


class LookupTable(BaseClass):
    def create(self, tablename, filename, importcontent=True):
        self.TableName = tablename
        self.FileName = os.path.basename(filename)

        if importcontent:
            self.import_content(filename)

    def import_content(self, filename):
        with open(filename, "r") as f:
            self._FileContent = f.readlines()


class Attachment(BaseClass):
    def create(self, name, filename, importcontent=True):
        self.AttachmentId = str(uuid.uuid4())
        self.Name = name

        if importcontent:
            self.import_content(filename)

    def import_content(self, filename):
        with open(filename, "rb") as f:
            self._FileName = os.path.basename(filename)
            self._FileContent = f.read()
            self._ImageType = imghdr.what(filename)
            self.ContentId = hashlib.sha1(self._FileContent).hexdigest().upper()


class Translation(BaseClass):
    def create(self, name, filename, importcontent=True):
        self.Id = str(uuid.uuid4())
        self.Name = name

        if importcontent:
            self.Id = str(uuid.uuid4())
            self.import_content(filename)

    def import_content(self, filename):
        with open(filename, "rb") as f:
            self._FileContent = f.read()


CLASS_DICT = {
    "Group": Group,
    "Answer": AnswerFactory,
    "AreaQuestion": QuestionFactory,
    "Attachment": Attachment,
    "AudioQuestion": QuestionFactory,
    "Question": QuestionFactory,
    "QuestionnaireDocument": Questionnaire,
    "QuestionProperties": QuestionProperties,
    "StaticText": StaticText,
    "DateTimeQuestion": QuestionFactory,
    "GpsCoordinateQuestion": QuestionFactory,
    "MultimediaQuestion": QuestionFactory,
    "MultyOptionsQuestion": MultyOptionsQuestion,
    "NumericQuestion": NumericQuestion,
    "QRBarcodeQuestion": QuestionFactory,
    "SingleQuestion": SingleQuestion,
    "TextListQuestion": TextListQuestion,
    "TextQuestion": TextQuestion,
    "Translation": Translation,
    "ValidationCondition": ValidationCondition,
    "Variable": Variable,
    CLASSTYPE_LOOKUPTABLE: MyDict,
    CLASSTYPE_MACRO: MyDict,
    "Macro": Macro,
    "MyDict": MyDict,
    "LookupTable": LookupTable,
}




from .base import HQBase
from .models import ExportJob
from .utils import parse_qidentity


class ExportApi(HQBase):
    _apiprefix: str = "/api/v2/export"

    def get_list(self, export_type=None, interview_status=None,
                 questionnaire_identity=None, export_status=None,
                 has_file=None):
        path = self.url
        params = {
            "exportType": export_type,
            "interviewStatus": interview_status,
            "questionnaireIdentity": parse_qidentity(questionnaire_identity),
            "exportStatus": export_status,
            "hasFile": has_file,
        }
        r = self._make_call('get', path, params=params)
        for item in r:
            yield ExportJob.from_dict(item)

    def get(self, questionnaire_identity: str, export_path: str = "", export_type: str = "Tabular"):
        """Downloads latest available export file

        Parameters
        ----------
        questionnaire_identity : str
            Questionnaire id in format QuestionnaireGuid$Version

        export_path: str
            Path to save the downloaded file

        export_type: str
            Format of the export data: ``Tabular``, ``STATA``, ``SPSS``, ``Binary``, ``DDI``, ``Paradata``
        """

        ret_list = self.get_list(
            export_type=export_type,
            questionnaire_identity=parse_qidentity(questionnaire_identity),
            export_status='Completed',
            has_file='true')
        try:
            download_link = next(ret_list).download_link
            return self._make_call('get', download_link, filepath=export_path, stream=True)
        except StopIteration:
            pass

    def get_info(self, job_id: int) -> ExportJob:
        path = self.url + '/{}'.format(job_id)
        return ExportJob.from_dict(self._make_call('get', path))

    def start(self, export_job: ExportJob) -> ExportJob:
        path = self.url
        return ExportJob.from_dict(self._make_call("post", path, json=export_job.to_json()))

    def cancel(self, job_id: int) -> None:
        response = self.get_info(job_id)
        if response.export_status == "Running":
            path = self.url + '{}'.format(job_id)
            _ = self._make_call('delete', path)
        else:
            print('No running export process was found')



from .base import HQBase
from .designer import import_questionnaire_json
from .models import QuestionnaireListItem


class QuestionnairesApi(HQBase):
    _apiprefix = "/api/v1/questionnaires"

    def get_list(self):
        path = self.url
        page_size = 10
        page = 1
        total_count = 11
        params = {
            'offset': page,
            'limit': page_size
        }
        while page * page_size < total_count:
            params['page'] = page
            r = self._make_call('get', path, params=params)
            if 'TotalCount' in r:
                total_count = r['TotalCount']
                for item in r['Questionnaires']:
                    yield QuestionnaireListItem.from_dict(item)
            else:
                yield from ()
            page += 1

    def statuses(self):
        path = self.url + '/statuses'
        return self._make_call('get', path)

    def document(self, id, version):
        path = self.url + '/{}/{}/document'.format(id, version)
        return self._make_call('get', path, parser=import_questionnaire_json)

    def interviews(self, id, version):
        path = self.url + '/{}/{}/interviews'.format(id, version)
        return self._make_call('get', path)

    def update_recordaudio(self, id, version):
        """POST /api/v1/questionnaires/{id}/{version}/recordAudio"""
        pass



import os

from setuptools import setup

packages = [
    'ssaw',
    'ssaw.designer',
]

requires = [
    'requests',
    'sgqlc',
]

test_requirements = [
    'flake8',
    'flake8-import-order',
    'pytest',
    'pytest-cov',
    'vcrpy',
]

here = os.path.abspath(os.path.dirname(__file__))

about = {}
with open(os.path.join(here, 'ssaw', '__about__.py'), 'r', encoding='utf-8') as f:
    exec(f.read(), about)

with open('README.md', 'r', encoding='utf-8') as f:
    readme = f.read()

classifiers = [
    'Development Status :: 4 - Beta',
    'Environment :: Console',
    'Environment :: Web Environment',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Programming Language :: Python :: 3 :: Only',
    'Programming Language :: Python :: Implementation :: PyPy',
]

setup(
    name=about['__title__'],
    version=about['__version__'],
    classifiers=classifiers,
    packages=packages,
    install_requires=requires,
    author=about['__author__'],
    author_email=about['__author_email__'],
    description=about['__description__'],
    long_description=readme,
    long_description_content_type='text/markdown',
    license="",
    keywords="Survey Solutions",
    url=about['__url__'],
    tests_require=test_requirements,
)
